import * as THREE from 'https://unpkg.com/three@v0.150.0/build/three.module.js';

var width = 800;
var height = 500;

// This creates a WebGL texture that we can render our depth image on
var depthTexture = new THREE.WebGLRenderTarget(width, height, { minFilter: THREE.LinearFilter, magFilter: THREE.NearestFilter, format: THREE.RGBFormat });
// We also create a new scene showing the background objects
var depthScene;

var renderer, scene, camera;
var chopper;
var hangar;

var smokeTexture;
var smokeParticles = [];

var cubeVertexShader, cubeFragmentShader;
var smokeVertexShader, smokeFragmentShader;
var depthVertexShader, depthFragmentShader;

/**
 * Function to tell us the milliseconds
 */
function millis() {
    return (new Date()).getTime();
}

/**
 * Converts degrees to radians
 */
function toRad(degree) {
    return Math.PI * 2 * degree / 360;
}

window.onLoad = async () => {
    var canvasContainer = document.getElementById('myCanvasContainer');

    // Load in the shaders  await (await fetch('vertex.glsl')).text();
    cubeVertexShader = await (await fetch('vertex_chopper.glsl')).text();
    cubeFragmentShader = await (await fetch('fragment_chopper.glsl')).text();

    smokeVertexShader = await (await fetch('vertex_smoke.glsl')).text();
    smokeFragmentShader = await (await fetch('fragment_smoke.glsl')).text();

    depthVertexShader = await (await fetch('vertex_depth.glsl')).text();
    depthFragmentShader = await (await fetch('fragment_depth.glsl')).text();

    // Create a renderer
    renderer = new THREE.WebGLRenderer();
    renderer.setSize(width, height);
    canvasContainer.appendChild(renderer.domElement);

    // Create two scene, one for background detph buffer another one for actual rendering
    scene = new THREE.Scene();
    depthScene = new THREE.Scene();

    // We will use a perspective camera, with FOV 80 degrees, correct aspect ratio, near plane at distance 1, far plane at distance 1000
    camera = new THREE.PerspectiveCamera(80, width / height, 1, 30);
    camera.position.z = 15;
    camera.position.y = 2;
    camera.up = new THREE.Vector3(0, 1, 0);
    camera.lookAt(new THREE.Vector3(0, 0, 0));
    scene.add(camera);
    depthScene.add(camera);

    addHangar();
    chopper = addChopper();

    var loader = new THREE.TextureLoader();
    loader.setCrossOrigin('Anonymous')
    loader.setPath('http://cglearn.eu/files/course/8/textures/');
    loader.load('smoke.png', onTextureLoaded);
}

function draw() {
    requestAnimationFrame(draw);

    // --Task-- Copy the rotation code from the previous task here.		

    // --Task-- Animate smoke particles by rotating them along z axis
    // NB! to obtain more variation modify your rotation such that:
    //	 1. Some particles rotate clockwise and others counter clockwise
    //	 2. Starting rotation varies from each particle

    // This renders background depth to the texture depthTexture
    renderer.setRenderTarget(depthTexture);
    renderer.render(depthScene, camera);
    renderer.setRenderTarget(null);

    // We render our main scene to the screen
    renderer.render(scene, camera);

    // For testing you can render depth texture to screen by using:
    // renderer.render(depthScene, camera);
}

function onTextureLoaded(texture) {
    smokeTexture = texture;
    addSmoke(texture);
    draw();
}

function addSmoke(texture) {
    var halfPi = Math.PI / 2;

    var geometry = new THREE.PlaneGeometry(10, 10, 1);
    var material = createSmokeShaderMaterial(texture);

    // -- Task -- right now only 1 smoke mesh is added to the scene
    //		 modify this code in such way that it fills the bottom of the hangar with smoke meshes.
    //	 NB! Do not try to prevent intersections with hangar walls.
    //		 Removing this visual artifact will be done later in this task.
    var smoke = new THREE.Mesh(geometry, material);
    smoke.position.set(0, -7, 0);
    scene.add(smoke);
    smokeParticles.push(smoke);
}

function createSmokeShaderMaterial(texture) {

    var material = new THREE.ShaderMaterial({
        // We'll use three.js built-in UV mapping in this shader.
        uniforms: {
            colorTexture: {
                type: 't',
                value: texture
            }
            // -- Task -- Send following uniforms to the fragment shader
            //	 1. hangar depth texture
            //	 2. near plane distance
            //	 3. far plane distance
            //	 4. window size (you will need it, try to figure out why)
        },
        vertexShader: smokeVertexShader,
        fragmentShader: smokeFragmentShader
        // -- Task -- allow transparency
    });

    return material;
}

function createDepthMaterial() {

    return new THREE.ShaderMaterial({
        uniforms: {
            // -- Task -- Send near and far plane distances to the fragment shader
        },
        vertexShader: depthVertexShader,
        fragmentShader: depthFragmentShader
    });
}

/**
 * Create and add the hangar to the scene.
 * Hangar currently consists of 5 walls which have different colors.
 */
function addHangar() {
    hangar = new THREE.Object3D();
    var halfPi = Math.PI / 2;

    var leftWall = createWall(0x555555);
    leftWall.position.set(-10, 0, 0);
    leftWall.rotation.set(0, halfPi, 0);
    hangar.add(leftWall);

    var rightWall = createWall(0x333333);
    rightWall.position.set(10, 0, 0);
    rightWall.rotation.set(0, -halfPi, 0);
    hangar.add(rightWall);

    var backWall = createWall(0x444444);
    backWall.position.set(0, 0, -10);
    hangar.add(backWall);

    var ceiling = createWall(0x111111);
    ceiling.position.set(0, 10, 0);
    ceiling.rotation.set(halfPi, 0, 0);
    hangar.add(ceiling);

    var floor = createWall(0x222222);
    floor.position.set(0, -10, 0);
    floor.rotation.set(-halfPi, 0, 0);

    hangar.add(floor);

    scene.add(hangar);

    var newHangar = hangar.clone();
    for (var i = 0; i < newHangar.children.length; i++) {
        newHangar.children[i].material = createDepthMaterial();
    }

    depthScene.add(newHangar);
}

/**
 * This function creates and adds the chopper.
 * --Task-- Copy the code from the previous task here.
 */
function addChopper() {
    var chopper = new THREE.Mesh();

    // --Task-- Copy the chopper code here

    return chopper;
}

/**
 * Creates a cube.
 * --Task-- Finish this function.
 */
function createCube(colorCode) {
    var geometry = new THREE.BoxGeometry(2, 2, 2);

    // --Task-- Copy the cube material code (from task shader chopper) here

    return cube;
}

/**
 * Creates a wall using the plane geometry.
 */
function createWall(colorCode) {
    var geometry = new THREE.PlaneGeometry(20, 20, 1);
    var material = new THREE.MeshBasicMaterial({ color: colorCode });
    var wall = new THREE.Mesh(geometry, material);

    return wall;
}